package java.text;

import sun.text.ComposedCharIter;
import sun.text.normalizer.NormalizerImpl;
import java.util.Vector;
import sun.text.UCompactIntArray;
import sun.text.IntHashtable;

final class RBTableBuilder
{
    static final int CHARINDEX = 1879048192;
    private static final int IGNORABLEMASK = 65535;
    private static final int PRIMARYORDERINCREMENT = 65536;
    private static final int SECONDARYORDERINCREMENT = 256;
    private static final int TERTIARYORDERINCREMENT = 1;
    private static final int INITIALTABLESIZE = 20;
    private static final int MAXKEYSIZE = 5;
    private RBCollationTables.BuildAPI tables;
    private MergeCollation mPattern;
    private boolean isOverIgnore;
    private char[] keyBuf;
    private IntHashtable contractFlags;
    private boolean frenchSec;
    private boolean seAsianSwapping;
    private UCompactIntArray mapping;
    private Vector<Vector<EntryPair>> contractTable;
    private Vector<int[]> expandTable;
    private short maxSecOrder;
    private short maxTerOrder;
    
    public RBTableBuilder(RBCollationTables.BuildAPI tables) {
        this.tables = null;
        this.mPattern = null;
        this.isOverIgnore = false;
        this.keyBuf = new char[5];
        this.contractFlags = new IntHashtable(100);
        this.frenchSec = false;
        this.seAsianSwapping = false;
        this.mapping = null;
        this.contractTable = null;
        this.expandTable = null;
        this.maxSecOrder = 0;
        this.maxTerOrder = 0;
        this.tables = tables;
    }
    
    public void build(String canonicalDecomposeWithSingleQuotation, int n) throws ParseException {
        if (canonicalDecomposeWithSingleQuotation.length() == 0) {
            throw new ParseException("Build rules empty.", 0);
        }
        this.mapping = new UCompactIntArray(-1);
        canonicalDecomposeWithSingleQuotation = NormalizerImpl.canonicalDecomposeWithSingleQuotation(canonicalDecomposeWithSingleQuotation);
        this.mPattern = new MergeCollation(canonicalDecomposeWithSingleQuotation);
        int increment = 0;
        for (int i = 0; i < this.mPattern.getCount(); ++i) {
            PatternEntry item = this.mPattern.getItemAt(i);
            if (item != null) {
                String s = item.getChars();
                if (s.length() > 1) {
                    switch (s.charAt(s.length() - 1)) {
                        case '@': {
                            this.frenchSec = true;
                            s = s.substring(0, s.length() - 1);
                            break;
                        }
                        case '!': {
                            this.seAsianSwapping = true;
                            s = s.substring(0, s.length() - 1);
                            break;
                        }
                    }
                }
                increment = this.increment(item.getStrength(), increment);
                String extension = item.getExtension();
                if (extension.length() != 0) {
                    this.addExpandOrder(s, extension, increment);
                }
                else if (s.length() > 1) {
                    int char1 = s.charAt(0);
                    if (Character.isHighSurrogate((char)char1) && s.length() == 2) {
                        this.addOrder(Character.toCodePoint((char)char1, s.charAt(1)), increment);
                    }
                    else {
                        this.addContractOrder(s, increment);
                    }
                }
                else {
                    this.addOrder((int)s.charAt(0), increment);
                }
            }
        }
        this.addComposedChars();
        this.commit();
        this.mapping.compact();
        this.tables.fillInTables(this.frenchSec, this.seAsianSwapping, this.mapping, this.contractTable, this.expandTable, this.contractFlags, this.maxSecOrder, this.maxTerOrder);
        return;
    }
    
    private void addComposedChars() throws ParseException {
        ComposedCharIter composedCharIter = new ComposedCharIter();
        int next;
        while ((next = composedCharIter.next()) != -1) {
            if (this.getCharOrder(next) != -1) {
                continue;
            }
            String decomposition = composedCharIter.decomposition();
            if (decomposition.length() == 1) {
                int charOrder2 = this.getCharOrder((int)decomposition.charAt(0));
                if (charOrder2 == -1) {
                    continue;
                }
                this.addOrder(next, charOrder2);
            }
            else if (decomposition.length() == 2 && Character.isHighSurrogate(decomposition.charAt(0))) {
                int charOrder = this.getCharOrder(decomposition.codePointAt(0));
                if (charOrder == -1) {
                    continue;
                }
                this.addOrder(next, charOrder);
            }
            else {
                int contractOrder = this.getContractOrder(decomposition);
                if (contractOrder != -1) {
                    this.addOrder(next, contractOrder);
                }
                else {
                    boolean n = true;
                    int i = 0;
                    while (i < decomposition.length()) {
                        if (this.getCharOrder((int)decomposition.charAt(i)) == -1) {
                            n = false;
                            break;
                        }
                        ++i;
                    }
                    if (!(n == 0)) {
                        this.addExpandOrder(next, decomposition, -1);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    private final void commit() {
        if (this.expandTable != null) {
            for (int i = 0; i < this.expandTable.size(); ++i) {
                int[] array = (int[])this.expandTable.elementAt(i);
                for (int j = 0; j < array.length; ++j) {
                    int n = array[j];
                    if (n < 2113929216 && n > 1879048192) {
                        int n2 = n - 1879048192;
                        int charOrder = this.getCharOrder(n2);
                        if (charOrder == -1) {
                            array[j] = (0xFFFF & n2);
                        }
                        else {
                            array[j] = charOrder;
                        }
                    }
                }
            }
        }
    }
    
    private final int increment(int n, int n2) {
        switch (n) {
            case 0: {
                n2 += 65536;
                n2 &= 0xFFFF0000;
                this.isOverIgnore = true;
                break;
            }
            case 1: {
                n2 += 256;
                n2 &= 0xFFFFFF00;
                if (!this.isOverIgnore) {
                    ++this.maxSecOrder;
                    break;
                }
                break;
            }
            case 2: {
                ++n2;
                if (!this.isOverIgnore) {
                    ++this.maxTerOrder;
                    break;
                }
                break;
            }
        }
        return n2;
    }
    
    private final void addOrder(int n, int value) {
        if (this.mapping.elementAt(n) >= 2130706432) {
            int chars = 1;
            if (Character.isSupplementaryCodePoint(n)) {
                chars = Character.toChars(n, this.keyBuf, 0);
            }
            else {
                this.keyBuf[0] = (char)n;
            }
            this.addContractOrder(new String(this.keyBuf, 0, chars), value);
        }
        else {
            this.mapping.setElementAt(n, value);
        }
    }
    
    private final void addContractOrder(String s, int n) {
        this.addContractOrder(s, n, true);
    }
    
    private final void addContractOrder(String str, int value, boolean b) {
        if (this.contractTable == null) {
            this.contractTable = new Vector(20);
        }
        int codePoint = str.codePointAt(0);
        int element = this.mapping.elementAt(codePoint);
        Vector contractValuesImpl = this.getContractValuesImpl(element - 2130706432);
        if (contractValuesImpl == null) {
            int value2 = 2130706432 + this.contractTable.size();
            contractValuesImpl = new Vector(20);
            this.contractTable.addElement(contractValuesImpl);
            contractValuesImpl.addElement(new EntryPair(str.substring(0, Character.charCount(codePoint)), element));
            this.mapping.setElementAt(codePoint, value2);
        }
        int entry = RBCollationTables.getEntry(contractValuesImpl, str, b);
        if (entry != -1) {
            ((EntryPair)contractValuesImpl.elementAt(entry)).value = value;
        }
        else if (str.length() > ((EntryPair)contractValuesImpl.lastElement()).entryName.length()) {
            contractValuesImpl.addElement(new EntryPair(str, value, b));
        }
        else {
            contractValuesImpl.insertElementAt(new EntryPair(str, value, b), contractValuesImpl.size() - 1);
        }
        if (b && str.length() > 1) {
            this.addContractFlags(str);
            this.addContractOrder(new StringBuffer(str).reverse().toString(), value, false);
        }
    }
    
    private int getContractOrder(String s) {
        int value = -1;
        if (this.contractTable != null) {
            Vector<EntryPair> contractValues = this.getContractValues(s.codePointAt(0));
            if (contractValues != null) {
                int entry = RBCollationTables.getEntry(contractValues, s, true);
                if (entry != -1) {
                    value = ((EntryPair)contractValues.elementAt(entry)).value;
                }
            }
        }
        return value;
    }
    
    private final int getCharOrder(int index) {
        int n = this.mapping.elementAt(index);
        if (n >= 2130706432) {
            n = ((EntryPair)this.getContractValuesImpl(n - 2130706432).firstElement()).value;
        }
        return n;
    }
    
    private Vector<EntryPair> getContractValues(int index) {
        return this.getContractValuesImpl(this.mapping.elementAt(index) - 2130706432);
    }
    
    private Vector<EntryPair> getContractValuesImpl(int index) {
        if (index >= 0) {
            return (Vector)this.contractTable.elementAt(index);
        }
        return null;
    }
    
    private final void addExpandOrder(String s, String s2, int n) throws ParseException {
        int addExpansion = this.addExpansion(n, s2);
        if (s.length() > 1) {
            int char1 = s.charAt(0);
            if (Character.isHighSurrogate((char)char1) && s.length() == 2) {
                int char2 = s.charAt(1);
                if (Character.isLowSurrogate((char)char2)) {
                    this.addOrder(Character.toCodePoint((char)char1, (char)char2), addExpansion);
                }
            }
            else {
                this.addContractOrder(s, addExpansion);
            }
        }
        else {
            this.addOrder((int)s.charAt(0), addExpansion);
        }
    }
    
    private final void addExpandOrder(int n, String s, int n2) throws ParseException {
        this.addOrder(n, this.addExpansion(n2, s));
    }
    
    private int addExpansion(int n, String s) {
        if (this.expandTable == null) {
            this.expandTable = new Vector(20);
        }
        int n2 = n != -1;
        int[] obj = new int[s.length() + n2];
        if (n2 == 1) {
            obj[0] = n;
        }
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            int codePoint;
            if (Character.isHighSurrogate((char)char1)) {
                if (++i != s.length()) {
                    int char2;
                    if (!Character.isLowSurrogate((char)(char2 = s.charAt(i)))) {
                        break;
                    }
                    codePoint = Character.toCodePoint((char)char1, (char)char2);
                }
                else {
                    break;
                }
            }
            else {
                codePoint = char1;
            }
            int charOrder = this.getCharOrder(codePoint);
            if (charOrder != -1) {
                obj[n2++] = charOrder;
            }
            else {
                obj[n2++] = 1879048192 + codePoint;
            }
        }
        if (n2 < obj.length) {
            int[] array = new int[n2];
            while (--n2 >= 0) {
                array[n2] = obj[n2];
            }
            obj = array;
        }
        int n3 = 2113929216 + this.expandTable.size();
        this.expandTable.addElement(obj);
        return n3;
    }
    
    private void addContractFlags(String s) {
        for (int length = s.length(), i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            char1 = (Character.isHighSurrogate((char)char1) ? Character.toCodePoint((char)char1, s.charAt(++i)) : char1);
            this.contractFlags.put(char1, 1);
        }
    }
}
